Package: HPZoneAPI
Type: Package
Title: 'HPZone' API Interface
Version: 1.3.0
Authors@R: person("Aart", "Dijkstra", email="a.dijkstra@ggdnog.nl", role=c("aut", "cre", "cph")) 
Maintainer: Aart Dijkstra <a.dijkstra@ggdnog.nl>
Description: Package that simplifies the use of the 'HPZone' API. Most of the annoying and labor-intensive parts of the interface are handled by wrapper functions. Note that the API and its details are not publicly available. Information can be found at <https://www.ggdghorkennisnet.nl/groep/726-platform-infectieziekte-epidemiologen/documenten/map/9609> for those with access. 
URL: https://github.com/ggdatascience/HPZoneAPI
BugReports: https://github.com/ggdatascience/HPZoneAPI/issues
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Imports: dplyr, httr2, jsonlite, keyring, lubridate, magrittr, readxl,
        rstudioapi, safer, stringr
Depends: R (>= 2.10)
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-09 10:07:26 UTC; a.dijkstra
Author: Aart Dijkstra [aut, cre, cph]
Repository: CRAN
Date/Publication: 2026-04-09 10:30:02 UTC
Built: R 4.5.2; ; 2026-04-09 10:56:19 UTC; unix

# Generated by roxygen2: do not edit by hand

export(HPZone_convert_dates)
export(HPZone_make_valid)
export(HPZone_necessary_scope)
export(HPZone_request)
export(HPZone_request_paginated)
export(HPZone_request_query)
export(HPZone_request_raw)
export(HPZone_setup)
export(HPZone_store_credentials)
export(test_HPZone_token)
importFrom(dplyr,coalesce)
importFrom(httr2,oauth_client)
importFrom(httr2,oauth_flow_client_credentials)
importFrom(httr2,req_body_raw)
importFrom(httr2,req_headers)
importFrom(httr2,req_oauth_client_credentials)
importFrom(httr2,req_perform)
importFrom(httr2,request)
importFrom(jsonlite,fromJSON)
importFrom(keyring,key_get)
importFrom(keyring,key_list)
importFrom(keyring,key_set_with_value)
importFrom(lubridate,ymd_hms)
importFrom(magrittr,"%>%")
importFrom(readxl,read_xlsx)
importFrom(rstudioapi,askForPassword)
importFrom(safer,decrypt_string)
importFrom(safer,encrypt_string)
importFrom(stats,setNames)
importFrom(stringr,fixed)
importFrom(stringr,regex)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_locate)
importFrom(stringr,str_sub)
importFrom(stringr,str_to_lower)

# HPZoneAPI

<!-- badges: start -->
<!-- badges: end -->

R-package om het aanroepen van de HPZone API makkelijker te maken. Er zijn enkele functies om de syntax van GraphQL wat te versimpelen, automatische naamcorrecties uit te voeren, en de benodigde scope te detecteren. 

## Installatie

Je kunt de huidige versie van HPZoneAPI installeren vanaf [GitHub](https://github.com/) met de volgende code:

``` r
# install.packages("devtools")
devtools::install_github("ggdatascience/HPZoneAPI")
```

## Beveiliging

De API key vereist een client ID en client secret voor gebruik. Om deze niet in je script te hoeven zetten wordt de keyring package gebruikt om de informatie op te slaan in de keyring van je OS. Dit betekent dat je per computer éénmalig de benodigde informatie moet invoeren, waarna deze beveiligd wordt opgeslagen. Hierbij wordt gebruik gemaakt van extra versleuteling, zodat andere processen, die in theorie ook bij deze keyring zouden kunnen, geen toegang hebben tot de data. Instellen gaat door het aanroepen van de functie *HPZone_store_credentials()*, welke interactief zal vragen om de benodigde informatie. Hierna kan *HPZone_setup()* zonder argumenten worden uitgevoerd.

Eenmalig:
``` r
HPZone_store_credentials()
```

Later gebruik in een script:
``` r
HPZone_request("iets")
```

## Gebruik

De package kan globaal gezien op twee manieren gebruikt worden: 1) als simpele wrapper om de aanroep van de API te vergemakkelijken, of 2) als uitbreiding op de workflow om gebruik in code te verkorten. De ontwikkeling is vooral gericht op scenario 2, waarbij een onderzoeker vanuit R makkelijk queries wil kunnen uitvoeren. Alle elementen die volgen uit technische specificaties van de API worden hiermee zoveel mogelijk uit handen genomen. Denk hierbij bijvoorbeeld aan een maximaal aantal rijen per aanvraag, het gebruik van quotes in een query, het opzetten van selectiecriteria, en het omgaan met datatypes. Hierdoor kan de volgende code bijvoorbeeld flink worden ingekort.

Handmatig:
``` r
i = 0
total = 500
while (i < total) {
  data = HPZone_request_raw(paste0('{"query": "{ cases (skip: ', i, ', take: 500, order: [ { Case_creation_date: ASC } ], where: { Status: { eq: \\"Open\\" } }) { items { Case_identifier, Date_of_onset }, totalCount } }" }'))
  data$data$cases$items$Date_of_onset = as.Date(data$data$cases$items$Date_of_onset)
  total = data$data$cases$totalCount
}
```

Met gebruik van de package:
``` r
data = HPZone_request("cases", "id", where=c("Status", "=", "Open"), order="Case_creation_date") %>%
  HPZone_convert_dates()
```
Of, als je toch liever handmatig de query schrijft:
``` r
data = HPZone_request_paginated('cases (order: [ { Case_creation_date: ASC } ], where: { Status: { eq: "Open" } }) { items { Case_identifier } }') %>%
  HPZone_convert_dates()
```

De *HPZone_request_xxx()* functies zijn ontworpen om zo makkelijk mogelijk te zijn in het dagelijks gebruik. Daarom zijn elementen die altijd uitgevoerd moeten worden hier standaard in verwerkt, en wordt er automatische tekencorrectie toegepast. Dit betekent dat het uitvoeren van de query korter kan dan in de handleiding aangegeven. Let hierbij vooral op het gebruik van de enkele quotes (') om de string te omvatten en het gebrek aan backslashes (\\) in de where-clausule. Deze worden binnen de functie automatisch toegevoegd. Verder is *sprintf()* geïntegreerd, en kun je dus gemakkelijk elementen dynamisch toevoegen.

## Query builder
Omdat GraphQL vol zit met eigenaardigheden, en dit zeker vanuit R voor onleesbare code kan zorgen, is er een speciale functie gemaakt die ook het ontwerpen van queries uit handen neemt. Hierbij hoef je niet meer na te denken over specifieke verschillen, zoals comparators (bijv. eq: vs. ==), hoef je niet meer te letten op specifieke spelling van velden ("case_creation" werkt net zo goed als "Case_creation_date"), en hoef je niet zelf de ingewikkelde logicastructuur toe te passen. Deze functie werkt als volgt:

`HPZone_request(endpoint, fields, where=NA, order=NA, verbose=F)`

Hierbij zijn er in vrijwel ieder argument extra trucjes verstopt. Je hoeft bijvoorbeeld niet zelf alle velden mee te geven, maar kunt ook volstaan met keywords.

### Endpoint
De naam hoeft niet correct te zijn; alle vormen die lijken op de juiste spelling worden automatisch omgezet.

### Fields
Kan een lijst met velden zijn, of één van de volgende keywords:

- "all" - alle beschikbare velden binnen het gewenste endpoint
- "basic" - een subset van velden die meestal gebruikt worden voor surveillance
- "standard" - alle velden die binnen de scope standard vallen
- "id"/"none" - alleen datum en HPZone ID (let op: het HPZone ID is anders dan de unique identifier!)

Bij zowel "basic" als "standard" is het tevens mogelijk om de lijst uit te breiden:
```r
HPZone_request("cases", c("basic", "Latitude", "Longitude"))
```

Handige tip: de beschikbare velden zijn tevens aanwezig in de variabele *HPZone_fields*.

### Where
Een lijst met de gewenste and/or-structuur, een vector met velden en hun waarden, of simpelweg een string met de gewenste query.
De lijst kan als volgt worden opgemaakt:
```r
HPZone_request("cases", "all", where=list("and"=list(c("Case_creation_date", "gte", "2025-01-01"), list("or"=c("Infection", "=", "Leptospirosis", "Infection", "==", "Malaria")))))
```
Hierbij worden alle gevallen opgehaald die aangemaakt zijn op of na 2025-01-01 en de infectie "Leptospirosis" of "Malaria" hebben. Dit voorbeeld laat tevens zien dat alle vormen van comparators worden geaccepteerd; eq (GraphQL), = (SQL), of == (R).
Als er geen AND of OR nodig is kan dit geheel worden overgeslagen en volstaat een vector met 3 argumenten:
```r
HPZone_request("cases", "all", where=c("Case_creation_date", "gte", "2025-01-01"))
```

Een serie van argumenten zonder logica wordt omgezet naar ANDs. Als je bijvoorbeeld alles in 2025 wil hebben kun je dit doen:
```r
HPZone_request("cases", "all", where=c("Case_creation_date", "gte", "2025-01-01", "Case_creation_date", "<", "2026-01-01"))
```
Wat identiek is aan dit:
```r
HPZone_request("cases", "all", where=list("and"=c("Case_creation_date", "gte", "2025-01-01", "Case_creation_date", "<", "2026-01-01")))
```


### Order
De gewenste sortering, in het format veldnaam=volgorde. Volgorde is niet verplicht, en wordt indien missend aangenomen als "ASC".
```r
# Let op het fout gespelde veld: Case_creation_date vs. creation_date
HPZone_request("cases", "all", where=c("creation_date", ">", "2025-09-01"), order=c("Infection", "Case_creation_date"="desc"))
```
Zoals deze code laat zien is ook correcte spelling van de volgorde niet nodig; desc wordt net zo goed geaccepteerd als DESC.

## Voorbeelden

### Casuistiek opvragen
Een simpel voorbeeld is het ophalen van de 50 meest recente casussen:

``` r
library(HPZoneAPI)

# let op: HPZone_setup() kan ook worden overgeslagen, als HPZone_store_credentials() eerder is uitgevoerd
HPZone_setup("client_id van je GGD", "client_secret van je GGD")
HPZone_request_query("cases(take: 50, order: [{ Case_creation_date: DESC }]) { items { Case_identifier }, totalCount }")
```

### Een volledig jaar ophalen
``` r
library(HPZoneAPI)

# let op: HPZone_setup() kan ook worden overgeslagen, als HPZone_store_credentials() eerder is uitgevoerd
HPZone_setup("client_id van je GGD", "client_secret van je GGD")
cur_year = 2025
fields = c("ABR", "Date_of_onset", "Infection")
HPZone_request("cases", fields, where=c("creation_date", ">=", paste0(cur_year, "-01-01"), "creation_date", "<", paste0(cur_year+1, "-01-01"))) %>%
  HPZone_convert_dates()
# of gelijkwaardig:
HPZone_request_paginated('cases(where: { and: [ { Case_creation_date: { gte: "%d-01-01" } }, { Case_creation_date: { lt: "%d-01-01" } } ]  }) { items { %s } }'), cur_year, cur_year+1, str_c(fields, collapse=", ")) %>%
  HPZone_convert_dates()
```

### Namen corrigeren
``` r
library(HPZoneAPI)

HPZone_make_valid(fields="date of onset")
# verwachte uitvoer: Date_of_onset
```

### Benodigde scope opvragen
``` r
library(HPZoneAPI)

HPZone_necessary_scope(c("Diagnosis", "Case_number", "Entered_by"))
# verwachte uitvoer: standard
```

## Beschikbare functies

- HPZone_request() - automatische query builder, welke eveneens automatisch de resultaten pagineert
- HPZone_request_query() - 'convenience wrapper', welke de query verkort en automatisch de data uit de respons haalt (maximaal 500 rijen)
- HPZone_request_paginated() - verdere verwerking van HPZone_request_query(), waarbij automatisch de volledige respons wordt opgehaald (geen maximum)
- HPZone_request_raw() - basisfunctie voor het opvragen van data, zonder verdere correctie of verwerking (maximaal 500 rijen)
- HPZone_convert_dates() - voegt een kolom toe (standaard Date_stat) die gebruikt kan worden als 'date for statistics' en wijzigt automatisch kolommen met een datum in het juiste datatype in R (let op: standaard alleen kolommen met date/datum/Received_on in de naam, maar dit gedrag is aan te passen, zie de documentatie bij ?HPZone_convert_dates)
- HPZone_make_valid() - accepteert een lijst met verkeerd gespelde namen en zet deze om naar de correcte notatie
- HPZone_necessary_scope() - accepteert een lijst met veldnamen en retourneert de benodigde scope
- test_HPZone_token() - controleert of de ingevoerde API key correct is
- HPZone_setup() - initialisatiefunctie die altijd als eerste moet worden uitgevoerd, tenzij er credentials zijn opgeslagen
- HPZone_store_credentials() - slaat de API credentials op een veilige manier op voor makkelijker hergebruik

#' Tests the supplied HPZone token against the token endpoint. Note that this does not check the type of access, only the functionality of the token.
#'
#' @return No return value.
#' @importFrom httr2 oauth_flow_client_credentials
#' @export
#'
#' @examples
#' \dontrun{
#' HPZone_setup()
#' # This will print the results.
#' test_HPZone_token()
#' }
test_HPZone_token = function () {
  check_setup()

  returns = list()

  if (!is.na(API_env$scope_standard)) {
    message("Testing the standard scope credentials...")
    token = httr2::oauth_flow_client_credentials(client=API_env$client)
    returns = append(returns, list("standard"=token))
  }

  if (!is.na(API_env$scope_extended)) {
    message("Testing the extended scope credentials...")
    token = httr2::oauth_flow_client_credentials(client=API_env$client)
    returns = append(returns, list("extended"=token))
  }

  if (is.na(API_env$scope_standard) && is.na(API_env$scope_extended)) {
    return("Neither scope was entered using HPZone_setup(). Terminating.")
  }

  return(returns)
}

#' Determines the necessary scope for a given list of fields.
#'
#' @param fields List of desired fields from a query.
#' @param endpoint The required endpoint.
#' @param resolve_fieldnames Whether or not to parse the supplied fields and convert them to valid HPZone field names. (By calling [HPZone_make_valid])
#'
#' @return The required scope as expected by the HPZone API, i.e. "standard" or "extended".
#' @export
#'
#' @examples
#' # these variables do not required an extended scope; desired response = standard
#' HPZone_necessary_scope(c("Diagnosis", "Case_number", "Entered_by"))
#' # Family_name requires the extended scope; desired response = extended
#' HPZone_necessary_scope(c("Diagnosis", "Case_number", "Family_name"))
HPZone_necessary_scope = function (fields, endpoint="Cases", resolve_fieldnames=TRUE) {
  if (is.na(endpoint) || is.null(endpoint)) {
    stop("No valid endpoint supplied.")
  }

  if (any(is.na(fields)) || any(is.null(fields))) {
    stop("No valid fields supplied.")
  }

  if (!endpoint %in% HPZone_fields$endpoint) {
    # case mismatch?
    endpoints = unique(HPZone_fields$endpoint)
    if (stringr::str_to_lower(endpoint) %in% stringr::str_to_lower(endpoints)) {
      endpoint = endpoints[stringr::str_to_lower(endpoint) == stringr::str_to_lower(endpoints)]
    } else {
      stop("Invalid endpoint supplied: ", endpoint)
    }
  }

  if (resolve_fieldnames) {
    fields = HPZone_make_valid(fields=fields)
  }

  required_scope = "standard"
  if (!all(HPZone_fields$scope_standard[which(HPZone_fields$field %in% fields & HPZone_fields$endpoint == endpoint)])) {
    required_scope = "extended"
  }

  return(required_scope)
}

test_that("necessary_scope returns correct scopes", {
  expect_equal(HPZone_necessary_scope(c("Diagnosis", "Case_number", "Entered_by")), "standard")
  expect_equal(HPZone_necessary_scope(c("Diagnosis", "Case_number", "Entered_by", "Family_name")), "extended")
})

#' Performs a HPZone request with the given parameters.
#'
#' Note that there are several helper functions that make the use of the API a lot simpler; specifically [HPZone_request()], [HPZone_request_paginated()].
#' This should only be used as a last resort; if manual GraphQL queries are required, [HPZone_request_query()] is highly advised instead.
#'
#' @param body A GraphQL query to send to the HPZone API, including all necessary bracketing and JSON-elements.
#' @param scope The desired scope; either standard or extended.
#'
#' @return An object containing the requested data points. This can be in different forms, depending on the request.
#' @export
#' @importFrom httr2 request
#' @importFrom httr2 req_headers
#' @importFrom httr2 req_body_raw
#' @importFrom httr2 req_oauth_client_credentials
#' @importFrom httr2 req_perform
#' @importFrom jsonlite fromJSON
#' @importFrom magrittr "%>%"
#'
#' @seealso [HPZone_request()], [HPZone_request_paginated()], [HPZone_request_query()]
#'
#' @examples
#' # Note the difference between the raw and convenience functions.
#' # These lines are equal:
#' \dontrun{
#' HPZone_request("cases", c("Case_creation_date", "Case_number"),
#'                where=c("Case_creation_date", ">=", "2025-01-01"))
#' HPZone_request_query(paste0('cases(where: { ',
#'                       'Case_creation_date: { gte: "2025-01-01" }',
#'                       '})',
#'                       '{ items { Case_creation_date, Case_number } }')
#'                     )
#' HPZone_request_raw(paste0('{"query": "{ cases(where: {',
#'                             'Case_creation_date: { gte: \\"2025-01-01\\" }',
#'                           '})',
#'                           '{ items { Case_creation_date, Case_number } }',
#'                           '}"}'))
#' }
HPZone_request_raw = function (body, scope=API_env$scope_standard) {
  check_setup()

  if (scope == "standard") scope = API_env$scope_standard
  if (scope == "extended") scope = API_env$scope_extended

  req = httr2::request(API_env$data_url) %>%
    httr2::req_headers(`content-type`="application/json",
                accept="application/json",
                scope=scope) %>%
    httr2::req_body_raw(body) %>%
    httr2::req_oauth_client_credentials(client=API_env$client) %>%
    httr2::req_perform()

  return(jsonlite::fromJSON(httr2::resp_body_json(req)))
}

#' Performs a HPZone request with the given parameters.
#'
#' This function is a convenience wrapper around [HPZone_request_raw()], to facilitate easier coding.
#' This function integrates [sprintf()] to allow for easier query design, quotes are automatically escaped, and the the result is unlisted to allow for easier access.
#' See the example for differences with [HPZone_request_raw()]. Note that results are *not* automatically paginated; use [HPZone_request_paginated()] for that.
#'
#' @param query A GraphQL query to send to the HPZone API. Note that only the actual query is required. (See the examples.)
#' @param ... Parameters to be passed to [sprintf()]. If empty, the body is not passed through [sprintf()].
#' @param scope The desired scope; either standard or extended.
#'
#' @return An object containing the requested data points. This can be in different forms, depending on the request, but is simplified as much as possible.
#' @export
#' @seealso [HPZone_request()], [HPZone_request_paginated()], [HPZone_convert_dates()]
#'
#' @examples
#' # Note the difference between the raw and convenience functions.
#' # These lines are equal:
#' \dontrun{
#' HPZone_request("cases", c("Case_creation_date", "Case_number"),
#'                where=c("Case_creation_date", ">=", "2025-01-01"))
#' HPZone_request_query(paste0('cases(where: {',
#'                               'Case_creation_date: { gte: "2025-01-01" }',
#'                             '}) {',
#'                               'items { Case_creation_date, Case_number }',
#'                              '}'))
#' HPZone_request_raw(paste0('{"query": "{ cases(where: {',
#'                             'Case_creation_date: { gte: \\"2025-01-01\\" }',
#'                           '}) {',
#'                             'items { Case_creation_date, Case_number }',
#'                           '} }"}'))
#' }
HPZone_request_query = function (query, ..., scope=API_env$scope_standard) {
  check_setup()

  if (scope == "standard") scope = API_env$scope_standard
  if (scope == "extended") scope = API_env$scope_extended

  if (...length() > 0) {
    query = sprintf(query, ...)
  }

  data = HPZone_request_raw(paste0('{"query": "{', gsub('"', '\\\\"', query) , '}"}'), scope=scope)
  # eliminate unnecessary listing; the format is something weird like data$cases$items
  while (length(data) == 1 && is.list(data)) {
    data = data[[1]]
  }

  # TODO: error handling
  return(data)
}

#' Performs a HPZone request with the given parameters.
#'
#' Convenience wrapper around [HPZone_request_query()]. This function automatically pulls the available number of records, rather than only the first 500.
#' Note that the current maximum for batching is 500 rows, so increasing n_max is not recommended.
#' This function is mainly intended for use when the query builder in [HPZone_request()] is insufficient. Usage of [HPZone_request()] is considerably easier otherwise.
#' An order clause is automatically added if not present to account for the lack of proper automatic sorting in the API.
#'
#' @param query A GraphQL query to send to the HPZone API. Note that keywords 'skip' and 'take' cannot be present in the query; this function will add them.
#' @param ... Parameters to be passed to sprintf(). If empty, the body is not passed through sprintf().
#' @param n_max Maximum number of entries to request per call.
#' @param scope The desired scope; either standard or extended.
#' @param verbose Can be used to display information about the looping request.
#'
#' @return A data frame containing all the responses gathered from the API. Only the items will be returned.
#' @export
#' @importFrom stringr str_locate
#' @importFrom stringr str_sub
#' @importFrom stringr str_detect
#' @importFrom stringr fixed
#' @importFrom dplyr coalesce
#'
#' @seealso [HPZone_request()], [HPZone_convert_dates()]
#'
#' @examples
#' \dontrun{
#' # Note the single quotes to facilitate double quote encapsulation for arguments.
#' HPZone_request_paginated(
#'   paste0('cases(where: {',
#'       'Case_creation_date: { gte: "2025-01-01" }',
#'     '}) {',
#'       'items { Case_creation_date, Case_number }',
#'      '}'))
#' # Or equal, making use of the sprintf integration:
#' startdate = "2025-01-01"
#' fields = c("Case_creation_date", "Case_number")
#' HPZone_request_paginated(
#'   'cases(where: { Case_creation_date: { gte: "%s" } }) { items { %s } }',
#'    startdate, stringr::str_c(fields, collapse=", ")
#' )
#' }
HPZone_request_paginated = function (query, ..., n_max=500, scope=API_env$scope_standard, verbose=FALSE) {
  check_setup()

  if (scope == "standard") scope = API_env$scope_standard
  if (scope == "extended") scope = API_env$scope_extended

  # find selector bracket, or lack thereof
  pos = stringr::str_locate(query, "(?<=[a-zA-Z]{1,10})\\s?(\\(|\\{)")
  query_char = trimws(stringr::str_sub(query, pos))
  # add order if lacking; required for successful pagination
  order_graphql = ""
  if (query_char == "(") {
    # selector bracket; insert before other selectors
    selectors = stringr::str_sub(query, start=pos[,1]+1, end=stringr::str_locate(query, stringr::fixed(")"))[,1]-1)
    if (stringr::str_detect(selectors, "skip|take")) {
      stop("'skip' or 'take' were present in the query. This is not supported in this function. Please use HPZone_request_query() or HPZone_request_raw() instead.")
    }

    if (!stringr::str_detect(selectors, "order")) {
      # add order if not present; required for successful pagination
      endpoint = stringr::str_sub(query, end=pos[1,1]-1)
      order_field = ifelse(endpoint == "enquiries", HPZone_make_valid("enquiries", "Received on"), HPZone_make_valid(endpoint, "creation_date"))

      order_graphql = paste0("order: [{ ", order_field, ": ASC }], ")
    }

    query = paste0(stringr::str_sub(query, end=pos[,1]), "[pageblock], ", order_graphql, stringr::str_sub(query, start=pos[,1]+1))
  } else if (query_char == "{") {
    # no selector bracket; insert
    # add order if not present; required for successful pagination
    endpoint = stringr::str_sub(query, end=pos[1,1]-1)
    order_field = ifelse(endpoint == "enquiries", HPZone_make_valid("enquiries", "Received on"), HPZone_make_valid(endpoint, "creation_date"))

    order_graphql = paste0(", order: [{ ", order_field, ": ASC }]")

    query = paste0(stringr::str_sub(query, end=pos[,1]-1), "([pageblock]", order_graphql, ")", stringr::str_sub(query, start=pos[,1]))
  } else {
    stop("No selector brackets nor opening curly brackets found. Make sure the query is properly formatted in GraphQL.")
  }

  # add totalCount to query, if not already present
  if (!stringr::str_detect(query, stringr::fixed("totalCount"))) {
    # this is pretty simple; it should be before the last curly bracket
    pos = unlist(stringr::str_locate_all(query, stringr::fixed("}")))
    pos = max(pos, na.rm=T)
    query = paste0(stringr::str_sub(query, end=pos-1), ", totalCount }")
  }

  if (...length() > 0) {
    query = sprintf(query, ...)
  }

  n_retrieved = 0
  n_present = Inf
  data_total = data.frame()
  if (verbose) message("Executing the following query with scope ", scope, ": ", query)
  while (n_retrieved < n_present) {
    if (verbose) message("Pulling up to ", n_max, " records, starting at index ", n_retrieved, " (expected number ", n_present, ")")
    data = HPZone_request_raw(paste0('{"query": "{', gsub('"', '\\\\"', sub("[pageblock]", paste0("take: ", n_max, ", skip: ", n_retrieved), query, fixed=T)) , '}"}'), scope=scope)
    # eliminate unnecessary listing; the format is something weird like data$cases$items
    while (length(data) == 1 && is.list(data)) {
      data = data[[1]]
    }

    n_present = data$totalCount
    data_total = rbind(data_total, data$items)
    n_retrieved = n_retrieved + dplyr::coalesce(nrow(data$items), 0)
  }

  # TODO: error handling
  return(data_total)
}

#' Performs a HPZone request with the given parameters.
#'
#' This function does most of the querybuilding for you, which allows for easier calling.
#' There are several shorthands for field selection, the request is automatically paginated, and the necessary scope is automatically detected from the desired fields.
#' Note that the take and skip elements are, by design, not present. If these are necessary, use [HPZone_request_query()] instead.
#'
#' @param endpoint The requested endpoint. Can be "cases", "situations", "enquiries", "contacts", or "actions". Case mismatch or spelling mismatch is automatically corrected with HPZone_make_valid().
#' @param fields A vector containing the required fields. Spelling is automatically corrected. Alternatively, the keywords "all" (all available fields), "basic" (usual fields for surveillance), "standard" (only fields in the standard scope), or "none"/"id" (only HPZone ID and date) can be used. "basic" and "standard" can be combined: c("basic", "Longitude", "Latitude")
#' @param where Either a vector containing pairs of 3 arguments, a literal query string, or a list outlining the selection criteria. See details.
#' @param order A vector of field=order pairs, e.g. c("Case_creation_date"="ASC"). If no order is supplied, ASC is assumed.
#' @param verbose Whether or not to display the calculated query and scope; useful for debugging query issues.
#'
#' @return An object containing the requested data points. This can be in different forms, depending on the request, but is simplified as much as possible.
#' @export
#' @importFrom stats setNames
#'
#' @seealso  [HPZone_request_paginated()], [HPZone_request_query()], [HPZone_make_valid()], [HPZone_convert_dates()]
#'
#' @details
#' The where clause can be specified in several formats. These can be:
#' * A literal string containing GraphQL. E.g. "Status: \{ eq: \"Open\" \}"
#' * A vector of strings containing three pairs: field, comparator, value. E.g. c("Status", "=", "Open"). Any usual R-style comparators are allowed and automatically translated, GraphQL comparators are left as-is.
#' * A list detailing the structure of the query, containing name-value pairs of keyword-selectors. This allows for complex and/or-structures, see the bottom example.
#'
#' @examples
#' \dontrun{
#' # These statements are equal:
#' HPZone_request("cases", "all",
#'                where=c("Case_creation_date", ">", "2025-10-01"))
#' HPZone_request("cases", "all",
#'                where=c("Case_creation_date", "gt", "2025-10-01"))
#'
#' # Selects cases after 2025-09-01, ordered by infection and then descending date.
#' HPZone_request("cases", "all",
#'                where=c("Case_creation_date", ">", "2025-09-01"),
#'                order=c("Infection", "Case_creation_date"="desc"))
#'
#' # Selects all cases which were registered after 2025-01-01 AND where Infection equals Leptospirosis.
#' HPZone_request("cases", "all",
#'                where=list("and"=c("Case_creation_date", "gte", "2025-01-01",
#'                                   "Infection", "=", "Leptospirosis")))
#' # Note that the default is AND, so this statement is equal:
#' HPZone_request("cases", "all",
#'                where=c("Case_creation_date", "gte", "2025-01-01",
#'                        "Infection", "=", "Leptospirosis"))
#'
#' # All cases after 2025-01-01 with either Leptospirosis or Malaria as infection.
#' # Note the nested list; adding a c() without a list() will warp the structure
#' # of the list and break everything.
#' HPZone_request("cases", "all",
#'                where=list(
#'                 "and"=list(c("Case_creation_date", "gte", "2025-01-01"),
#'                            list("or"=c("Infection", "=", "Leptospirosis",
#'                                        "Infection", "==", "Malaria"))
#'                           )))
#' }
HPZone_request = function (endpoint, fields, where=NA, order=NA, verbose=FALSE) {
  check_setup()

  # valid endpoint?
  if (length(endpoint) != 1) {
    stop("Only one endpoint can be supplied per request.")
  }
  endpoint = HPZone_make_valid(endpoints=endpoint)

  if (length(fields) == 1 && fields == "all") {
    fields = HPZone_fields$field[tolower(HPZone_fields$endpoint) == endpoint]
  } else if (any(fields == "basics") || any(fields == "basic")) {
    # basic subset for each endpoint
    fields = fields[!(fields == "basics" | fields == "basic")]
    fields = c(fields, HPZone_fields$field[tolower(HPZone_fields$endpoint) == endpoint & HPZone_fields$in_basic])
  } else if (any(fields == "standard")) {
    # basic subset for each endpoint
    fields = fields[!(fields == "standard")]
    fields = c(fields, HPZone_fields$field[tolower(HPZone_fields$endpoint) == endpoint & HPZone_fields$scope_standard])
  } else if (length(fields) == 1 && (fields == "id" || fields == "none")) {
    # only request IDs
    if (endpoint == "cases")
      fields = c("Case_number", "Case_creation_date")
    else if (endpoint == "situations")
      fields = c("Situation_number", "Situation_creation_date")
    else if (endpoint == "enquiries")
      fields = c("Enquiry_number", "Received_on")
    else if (endpoint == "actions")
      fields = c("Number", "Date_created")
    else if (endpoint == "contacts")
      fields = c("Contact_number", "Contact_creation_date")
  }
  if (length(fields) == 1) {
    fields = trimws(unlist(strsplit(fields, ",")))
  }
  fields = HPZone_make_valid(endpoint, fields)

  # determine scope
  scope = HPZone_necessary_scope(fields, endpoint)

  # the query is in this format:
  # [endpoint](where?, order?, take?, skip?) { items { [fields] }, totalCount? }
  # e.g. cases(where: { Case_creation_date: { gte: "2020-01-01" } }) { items { Gp, Family_name } }
  # take and skip are not supported in this function, as HPZone_request_paginated() will cycle through those
  selectors = c()
  if ((is.vector(where) && any(!is.na(where))) || !is.na(where)) {
    # where can be in several formats, and with several shorthands
    # this function allows:
    # - a literal string, e.g. "Infection: { eq: "Hepatitis B" }"
    # - a list of selectors, with the name as the logic operator and then a list of three logic elements;
    #   e.g.: list("and"=c("Infection", "==", "Hepatitis B", "Status", "==", "Open"))
    # - R-like syntax: where = "field comparator value", e.g. where = "Infection == "
    # list of selectors
    if (length(where) == 1 && is.character(where)) {
      if (is.null(names(where))) {
        # literal string, do nothing
      } else {
        # convert syntax
        where = parse_query_list(endpoint, where)
      }
    } else if (is.list(where) || is.vector(where)) {
      where = parse_query_list(endpoint, where)
    } else {
      stop("Unknown format for the where clause supplied. Consult the documentation.")
    }

    selectors = c(selectors, paste0("where: { ", where, " }"))
  }
  if ((is.vector(order) && any(!is.na(order))) || !is.na(order)) {
    if (is.null(names(order))) {
      # assume ASC
      order = stats::setNames(rep("ASC", length(order)), order)
    }
    if (any(names(order) == "")) {
      # some values are unnamed; flip name-value pair around
      indexes = names(order) == ""
      values = order[indexes]
      order[indexes] = rep("ASC", sum(indexes))
      names(order)[indexes] = values
    }

    names(order) = HPZone_make_valid(endpoint, fields=names(order))
    order = trimws(toupper(order))

    order_graphql = paste0("{ ", names(order), ": ", unname(order), " }")

    selectors = c(selectors, paste0("order: [", stringr::str_c(order_graphql, collapse=", "), "]"))
  }

  if (length(selectors) > 0) {
    query = paste0(endpoint, "(", stringr::str_c(selectors, collapse=", "), ") { items { ", stringr::str_c(fields, collapse=", "), " } }")
  } else {
    query = paste0(endpoint, " { items { ", stringr::str_c(fields, collapse=", "), " } }")
  }

  if (verbose) message("Executing the following query with scope ", scope, ": ", query)
  data = HPZone_request_paginated(query, scope=scope, verbose=verbose)

  return(data)
}

graphql_dictionary = c("="="eq", "=="="eq", "!="="neq", ">"="gt", ">="="gte", "<"="lt", "<="="lte", "%in%"="in",
                       "|"="or", "||"="or", "&"="and", "&&"="and")
convert_graphql = function (keyword) {
  keyword = tolower(trimws(keyword))
  if (keyword %in% names(graphql_dictionary)) {
    keyword = graphql_dictionary[keyword]
  }
  return(keyword)
}

parse_query_list = function (endpoint, where) {
  if (is.list(where)) {
    where_elements = c()
    for (i in 1:length(where)) {
      logic = names(where)[i]
      if (is.null(logic) || is.na(logic)) {
        logic = "and"
      } else {
        logic = tolower(logic)
      }

      elements = where[[i]]
      if (is.list(elements)) {
        graphql_elements = c()
        for (j in 1:length(elements)) {
          graphql_elements = c(graphql_elements, parse_query_list(endpoint, elements[j]))
        }
      } else if (is.character(elements) && length(elements) == 3) {
        graphql_elements = parse_query_list(endpoint, elements)
      } else if (is.character(elements) && length(elements) %% 3 == 0) {
        graphql_elements = c()
        for (j in seq(1, length(elements), 3)) {
          graphql_elements = c(graphql_elements, parse_query_list(endpoint, elements[j:(j+2)]))
        }
      } else {
        stop("Query elements should be in multiples of 3; c(\"fieldname\", \"==\", \"value\")")
      }

      if (logic != "" && length(graphql_elements) > 1) {
        graphql = paste0(logic, ": [", stringr::str_c(paste0("{ ", graphql_elements, " }"), collapse=", "), "]")
      } else {
        graphql = stringr::str_c(graphql_elements, collapse=", ")
      }

      where_elements = c(where_elements, graphql)
    }
    return(stringr::str_c(where_elements, collapse=", "))
  } else if (is.character(where) && length(where) == 3) {
    # in case of a numeric value, GraphQL wants no quotes
    if (grepl("^[0-9.,]+$", where[3]) || is.numeric(where[3]))
      return(paste0(HPZone_make_valid(endpoint, where[1]), ": { ", convert_graphql(where[2]), ": ", where[3], " }"))
    return(paste0(HPZone_make_valid(endpoint, where[1]), ": { ", convert_graphql(where[2]), ": \"", where[3], "\" }"))
  }
  else if (is.character(where) && length(where) %% 3 == 0) {
    return(parse_query_list(endpoint, list("and"=where)))
  }
}

# # C-style print functie
# printf = function (...) cat(paste(sprintf(...),"\n"))
#
# ## log levels
# ERR = 1
# WARN = 2
# MSG = 3
# DEBUG = 4
#
# # functie om (fout)meldingen weer te geven
# # middels het level kunnen verschillende niveau's worden aangegeven, waarbij een lager niveau
# # altijd wordt weergegeven (dus msg("test", WARN) wordt ook getoond bij log.level = 3)
# # - msg kan een object of een sprintf-style string zijn
# # - level moet een log level zijn
# # - overige argumenten worden doorgegeven aan sprintf
# msg = function (msg, ..., level = DEBUG) {
#   # alleen output geven als het ewenste level gelijk is of hoger dan de gegeven waarde
#   if (exists("log.level")) {
#     if (level > log.level) {
#       return()
#     }
#   }
#
#   # schrijf naar logbestand
#   if (log.save) {
#     if (is.character(msg)) {
#       cat(sprintf(paste0("[%s - %s] ", msg, "\n"), Sys.time(), deparse(substitute(level)), ...), file = "log.txt", append=T)
#     } else {
#       cat(sprintf(paste0("[%s - %s] ", msg, "\n"), Sys.time(), deparse(substitute(level))), file = "log.txt", append=T)
#     }
#   }
#
#   # geen gewenst level of level hoog genoeg; printen
#   if (level == ERR) {
#     if (is.character(msg))
#       stop(sprintf(msg, ...))
#     else
#       stop(msg)
#   } else if (level == WARN) {
#     if (is.character(msg))
#       message(sprintf(msg, ...))
#     else
#       message(msg)
#   } else if (level == MSG) {
#     if (is.character(msg))
#       message(sprintf(msg, ...))
#     else
#       message(msg)
#   } else {
#     if (is.character(msg))
#       printf(paste0(ifelse(level == DEBUG, "[DEBUG] ", ""), msg), ...)
#     else
#       print(msg)
#   }
# }

#' Converts all relevant columns in the dataset to Date types.
#'
#' Based on the available fields, a date for statistics is automatically generated, using the following logic:
#' If cases: Date_of_onset > Datum_melding_aan_de_ggd > Case_creation_date.
#' If situations: Start_date > Situation_creation_date
#' If enquiries: Received_on > Date_closed
#'
#' @param data A dataset returned by the API.
#' @param search Column names to search for. Default: anything containing 'date' or 'datum'.
#' @param statdate Desired column name for the date for statistics field.
#'
#' @return An equivalent data.frame with Date types in the correct columns. Additionally, a column called 'Date_stat' is added, see Details.
#' @export
#' @importFrom stringr str_detect
#' @importFrom stringr regex
#' @importFrom lubridate ymd_hms
#' @importFrom dplyr coalesce
#'
#' @examples
#' \dontrun{
#' HPZone_request("cases", "basic", where=c("Case_creation_date", ">", "2025-01-01")) |>
#'   HPZone_convert_dates()
#' }
HPZone_convert_dates = function (data, search="dat(e|um)|Received_on", statdate="Date_stat") {
  dataset = data
  if (is.list(data) && "items" %in% names(data)) {
    dataset = data$items
  }

  cols = which(stringr::str_detect(colnames(dataset), stringr::regex(search, ignore_case=T)))
  for (col in cols) {
    dataset[[col]] = suppressWarnings(lubridate::ymd_hms(dataset[[col]]))
  }

  # calculate statistics date
  if (any(c("Case_creation_date", "Date_of_onset", "Datum_melding_aan_de_ggd") %in% colnames(dataset))) {
    # cases
    dataset[, statdate] = dplyr::coalesce(dataset$Date_of_onset, dataset$Datum_melding_aan_de_ggd, dataset$Case_creation_date)
  } else if (any(c("Start_date", "Situation_creation_date") %in% colnames(dataset))) {
    # situations
    dataset[, statdate] = dplyr::coalesce(dataset$Start_date, dataset$Situation_creation_date)
  }
  else if (any(c("Received_on", "Date_closed") %in% colnames(dataset))) {
    # enquiries
    dataset[, statdate] = dplyr::coalesce(dataset$Received_on, dataset$Date_closed)
  }

  return(dataset)
}

API_env = new.env(parent = emptyenv())

API_env$init_run = F
API_env$token_url = "https://connect.govconext.nl/oidc/token"
API_env$data_url = "https://api.hpzone.nl:8899/Edie"
API_env$scope_standard = "standard"
API_env$scope_extended = "extended"
API_env$client_id = NA
API_env$client_secret = NA


#' Initialisation function to define API credentials. This function must be called before anything else, as the details supplied in this call are required for use.
#' Note that client_id and client_secret should preferably not be supplied in this call, but rather stored using [HPZone_store_credentials()].
#'
#' @param client_id Client ID as supplied by InFact.
#' @param client_secret Client secret as supplied by InFact.
#' @param standard Name of the standard scope. Default: "standard"
#' @param extended Name of the extended scope. Default: "extended"
#' @param token_url Address of the token server.
#' @param data_url Address of the data server.
#'
#' @return No return value.
#' @importFrom httr2 oauth_client
#' @importFrom keyring key_list
#' @importFrom keyring key_get
#' @importFrom safer decrypt_string
#' @export
#'
#' @examples
#' # Not recommended:
#' HPZone_setup("id", "secret")
#'
#' # Recommended:
#' \dontrun{
#' HPZone_store_credentials() # call once
#' HPZone_setup() # will automatically read stored credentials
#' }
HPZone_setup = function (client_id = NA, client_secret = NA, standard="standard", extended="extended",
                         token_url="https://connect.govconext.nl/oidc/token",
                         data_url="https://api.hpzone.nl:8899/Edie") {
  if (is.na(client_id) || is.na(client_secret)) {
    # check for stored credentials
    stored_keys = keyring::key_list()
    if (is.na(client_id) && "HPZone_client_id" %in% stored_keys$service) {
      client_id = safer::decrypt_string(keyring::key_get("HPZone_client_id"), key="Kdj(327KWpX%")
    }

    if (is.na(client_secret) && "HPZone_client_secret" %in% stored_keys$service) {
      client_secret = safer::decrypt_string(keyring::key_get("HPZone_client_secret"), key="Kdj(327KWpX%")
    }
  }

  if (is.na(client_id) || is.na(client_secret)) {
    stop("No client ID or secret supplied, and no previously stored credentials found. Use HPZone_store_credentials() if you want to store credentials.")
  }

  API_env$client_id = client_id
  API_env$client_secret = client_secret
  API_env$scope_standard = standard
  API_env$scope_extended = extended
  API_env$token_url = token_url
  API_env$data_url = data_url

  API_env$client = httr2::oauth_client(id=API_env$client_id, token_url=API_env$token_url, secret=API_env$client_secret, name="HPZone-API-R-package")

  API_env$init_run = T
}

#' Safely stores the HPZone API details, so you don't need to put them in your script. The default OS keyring backend is used through the keyring-package. Values are stored in an encrypted format to prevent harvesting by other applications.
#' Note that the actual values are not supplied as arguments, but requested using rstudioapi password prompts.
#'
#' @param client_id True / false: whether the client_id should be stored.
#' @param client_secret True / false: whether the client_secret should be stored.
#'
#' @return N/A
#' @export
#' @importFrom rstudioapi askForPassword
#' @importFrom keyring key_set_with_value
#' @importFrom safer encrypt_string
#'
#' @examples
#' \dontrun{
#' # simply execute this line to store credentials
#' HPZone_store_credentials()
#' # after use, setup can be ran without arguments:
#' HPZone_setup()
#' }
HPZone_store_credentials = function (client_id = TRUE, client_secret = TRUE) {
  if (client_id) {
    secret_id = rstudioapi::askForPassword("Please enter the client_id")
    keyring::key_set_with_value("HPZone_client_id", password=safer::encrypt_string(secret_id, key="Kdj(327KWpX%"))
  }

  if (client_secret) {
    secret_secret = rstudioapi::askForPassword("Please enter the client_secret")
    keyring::key_set_with_value("HPZone_client_secret", password=safer::encrypt_string(secret_secret, key="Kdj(327KWpX%"))
  }
}

check_setup = function () {
  if (API_env$init_run) {
    return(T)
  }
  if (!API_env$init_run) {
    # check for stored credentials
    stored_keys = keyring::key_list()
    if ("HPZone_client_id" %in% stored_keys$service && "HPZone_client_secret" %in% stored_keys$service) {
      HPZone_setup()
      return(T)
    } else {
      stop("The package has not yet been initialized. Run HPZone_setup() first.")
    }
  }
}

globalVariables("HPZone_fields")

#' Takes a list of fields or endpoints and corrects them. This allows for short hand usage without having to check the documentation, e.g. 'Date of onset' instead of 'Date_of_onset'
#'
#' @param endpoints A list of desired endpoints.
#' @param fields A list of desired fields.
#'
#' @return A list of properly formatted fields or endpoints.
#' @importFrom stringr str_to_lower
#' @importFrom stringr str_c
#' @export
#'
#' @examples
#' HPZone_make_valid("case") # should return "cases"
#' HPZone_make_valid(fields="case_creation") # should return "Case_creation_date"
#' HPZone_make_valid("case", fields=c("Family name", "Gp")) # should return c("Family_name", "Gp")
HPZone_make_valid = function (endpoints=NULL, fields=NULL) {
  if (!is.null(fields)) {
    possible_fields = HPZone_fields$field
    possible_fields_hr = HPZone_fields$field_hr
    if (!is.null(endpoints)) {
      endpoints = HPZone_make_valid(endpoints)
      possible_fields = HPZone_fields$field[stringr::str_to_lower(HPZone_fields$endpoint) %in% endpoints]
      possible_fields_hr = HPZone_fields$field_hr[stringr::str_to_lower(HPZone_fields$endpoint) %in% endpoints]
    }
    fields_valid = unlist(lapply(fields,  function (x) {
      index = grep(x, possible_fields, ignore.case=T)
      if (length(index) == 0) {
        index = grep(x, possible_fields_hr, ignore.case=T)
        if (length(index) == 0) {
          stop("Invalid field supplied: ", x)
        }
        if (length(index) > 1) {
          # if one field matches exactly, minus case sensitivity, that's the one we want
          # otherwise: error
          if (stringr::str_to_lower(x) %in% stringr::str_to_lower(possible_fields_hr[index]) &&
              sum(stringr::str_to_lower(x) == stringr::str_to_lower(possible_fields_hr[index])) == 1) {
            return(index[stringr::str_to_lower(x) == stringr::str_to_lower(possible_fields_hr[index])])
          }
          stop("Multiple fields match the supplied description (", x, "): ", stringr::str_c(possible_fields[index], collapse=", "))
        }
      }
      if (length(index) > 1) {
        # if one field matches exactly, minus case sensitivity, that's the one we want
        # otherwise: error
        if (stringr::str_to_lower(x) %in% stringr::str_to_lower(possible_fields[index]) &&
            sum(stringr::str_to_lower(x) %in% stringr::str_to_lower(possible_fields[index])) == 1) {
          return(index[stringr::str_to_lower(x) == stringr::str_to_lower(possible_fields[index])])
        }
        stop("Multiple fields match the supplied description (", x, "): ", stringr::str_c(possible_fields[index], collapse=", "))
      }
      return(index)
    }))
    return(possible_fields[fields_valid])
  } else if (!is.null(endpoints)) {
    possible_endpoints = str_to_lower(unique(HPZone_fields$endpoint))
    endpoints_valid = unlist(lapply(endpoints, function (x) {
      index = grep(x, possible_endpoints, ignore.case=T)
      if (length(index) != 1) stop("Invalid endpoint supplied: ", x)
      return(index)
    }))
    return(possible_endpoints[endpoints_valid])
  }
}

test_that("make_valid allows case mismatch and truncation in endpoints", {
  expect_equal(HPZone_make_valid(endpoints="case"), "cases")
  expect_equal(HPZone_make_valid(endpoints="cases"), "cases")
  expect_equal(HPZone_make_valid(endpoints=c("cases", "contact", "enq")), c("cases", "contacts", "enquiries"))
  expect_error(HPZone_make_valid(endpoints="casessss"))
})

test_that("make_valid allows case mismatch and truncation in fields", {
  expect_equal(HPZone_make_valid(fields="diagnosis"), "Diagnosis")
  expect_equal(HPZone_make_valid(fields="date of onset"), "Date_of_onset")
  expect_equal(HPZone_make_valid(fields=c("date of onset", "Datum_definitief")), c("Date_of_onset", "Datum_definitief_in_osiris"))
})

test_that("make_valid throws errors for ambiguous names", {
  expect_error(HPZone_make_valid(fields="diag"))
  expect_error(HPZone_make_valid(fields="date of on"))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HPZone_request.R
\name{HPZone_request}
\alias{HPZone_request}
\title{Performs a HPZone request with the given parameters.}
\usage{
HPZone_request(endpoint, fields, where = NA, order = NA, verbose = FALSE)
}
\arguments{
\item{endpoint}{The requested endpoint. Can be "cases", "situations", "enquiries", "contacts", or "actions". Case mismatch or spelling mismatch is automatically corrected with HPZone_make_valid().}

\item{fields}{A vector containing the required fields. Spelling is automatically corrected. Alternatively, the keywords "all" (all available fields), "basic" (usual fields for surveillance), "standard" (only fields in the standard scope), or "none"/"id" (only HPZone ID and date) can be used. "basic" and "standard" can be combined: c("basic", "Longitude", "Latitude")}

\item{where}{Either a vector containing pairs of 3 arguments, a literal query string, or a list outlining the selection criteria. See details.}

\item{order}{A vector of field=order pairs, e.g. c("Case_creation_date"="ASC"). If no order is supplied, ASC is assumed.}

\item{verbose}{Whether or not to display the calculated query and scope; useful for debugging query issues.}
}
\value{
An object containing the requested data points. This can be in different forms, depending on the request, but is simplified as much as possible.
}
\description{
This function does most of the querybuilding for you, which allows for easier calling.
There are several shorthands for field selection, the request is automatically paginated, and the necessary scope is automatically detected from the desired fields.
Note that the take and skip elements are, by design, not present. If these are necessary, use [HPZone_request_query()] instead.
}
\details{
The where clause can be specified in several formats. These can be:
* A literal string containing GraphQL. E.g. "Status: \{ eq: \"Open\" \}"
* A vector of strings containing three pairs: field, comparator, value. E.g. c("Status", "=", "Open"). Any usual R-style comparators are allowed and automatically translated, GraphQL comparators are left as-is.
* A list detailing the structure of the query, containing name-value pairs of keyword-selectors. This allows for complex and/or-structures, see the bottom example.
}
\examples{
\dontrun{
# These statements are equal:
HPZone_request("cases", "all",
               where=c("Case_creation_date", ">", "2025-10-01"))
HPZone_request("cases", "all",
               where=c("Case_creation_date", "gt", "2025-10-01"))

# Selects cases after 2025-09-01, ordered by infection and then descending date.
HPZone_request("cases", "all",
               where=c("Case_creation_date", ">", "2025-09-01"),
               order=c("Infection", "Case_creation_date"="desc"))

# Selects all cases which were registered after 2025-01-01 AND where Infection equals Leptospirosis.
HPZone_request("cases", "all",
               where=list("and"=c("Case_creation_date", "gte", "2025-01-01",
                                  "Infection", "=", "Leptospirosis")))
# Note that the default is AND, so this statement is equal:
HPZone_request("cases", "all",
               where=c("Case_creation_date", "gte", "2025-01-01",
                       "Infection", "=", "Leptospirosis"))

# All cases after 2025-01-01 with either Leptospirosis or Malaria as infection.
# Note the nested list; adding a c() without a list() will warp the structure
# of the list and break everything.
HPZone_request("cases", "all",
               where=list(
                "and"=list(c("Case_creation_date", "gte", "2025-01-01"),
                           list("or"=c("Infection", "=", "Leptospirosis",
                                       "Infection", "==", "Malaria"))
                          )))
}
}
\seealso{
[HPZone_request_paginated()], [HPZone_request_query()], [HPZone_make_valid()], [HPZone_convert_dates()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HPZoneAPI-package.R
\docType{package}
\name{HPZoneAPI-package}
\alias{HPZoneAPI}
\alias{HPZoneAPI-package}
\title{HPZoneAPI: 'HPZone' API Interface}
\description{
Package that simplifies the use of the 'HPZone' API. Most of the annoying and labor-intensive parts of the interface are handled by wrapper functions. Note that the API and its details are not publicly available. Information can be found at \url{https://www.ggdghorkennisnet.nl/groep/726-platform-infectieziekte-epidemiologen/documenten/map/9609} for those with access.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ggdatascience/HPZoneAPI}
  \item Report bugs at \url{https://github.com/ggdatascience/HPZoneAPI/issues}
}

}
\author{
\strong{Maintainer}: Aart Dijkstra \email{a.dijkstra@ggdnog.nl} [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test_HPZone_token.R
\name{test_HPZone_token}
\alias{test_HPZone_token}
\title{Tests the supplied HPZone token against the token endpoint. Note that this does not check the type of access, only the functionality of the token.}
\usage{
test_HPZone_token()
}
\value{
No return value.
}
\description{
Tests the supplied HPZone token against the token endpoint. Note that this does not check the type of access, only the functionality of the token.
}
\examples{
\dontrun{
HPZone_setup()
# This will print the results.
test_HPZone_token()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HPZone_request.R
\name{HPZone_request_raw}
\alias{HPZone_request_raw}
\title{Performs a HPZone request with the given parameters.}
\usage{
HPZone_request_raw(body, scope = API_env$scope_standard)
}
\arguments{
\item{body}{A GraphQL query to send to the HPZone API, including all necessary bracketing and JSON-elements.}

\item{scope}{The desired scope; either standard or extended.}
}
\value{
An object containing the requested data points. This can be in different forms, depending on the request.
}
\description{
Note that there are several helper functions that make the use of the API a lot simpler; specifically [HPZone_request()], [HPZone_request_paginated()].
This should only be used as a last resort; if manual GraphQL queries are required, [HPZone_request_query()] is highly advised instead.
}
\examples{
# Note the difference between the raw and convenience functions.
# These lines are equal:
\dontrun{
HPZone_request("cases", c("Case_creation_date", "Case_number"),
               where=c("Case_creation_date", ">=", "2025-01-01"))
HPZone_request_query(paste0('cases(where: { ',
                      'Case_creation_date: { gte: "2025-01-01" }',
                      '})',
                      '{ items { Case_creation_date, Case_number } }')
                    )
HPZone_request_raw(paste0('{"query": "{ cases(where: {',
                            'Case_creation_date: { gte: \\\\"2025-01-01\\\\" }',
                          '})',
                          '{ items { Case_creation_date, Case_number } }',
                          '}"}'))
}
}
\seealso{
[HPZone_request()], [HPZone_request_paginated()], [HPZone_request_query()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HPZone_make_valid.R
\name{HPZone_make_valid}
\alias{HPZone_make_valid}
\title{Takes a list of fields or endpoints and corrects them. This allows for short hand usage without having to check the documentation, e.g. 'Date of onset' instead of 'Date_of_onset'}
\usage{
HPZone_make_valid(endpoints = NULL, fields = NULL)
}
\arguments{
\item{endpoints}{A list of desired endpoints.}

\item{fields}{A list of desired fields.}
}
\value{
A list of properly formatted fields or endpoints.
}
\description{
Takes a list of fields or endpoints and corrects them. This allows for short hand usage without having to check the documentation, e.g. 'Date of onset' instead of 'Date_of_onset'
}
\examples{
HPZone_make_valid("case") # should return "cases"
HPZone_make_valid(fields="case_creation") # should return "Case_creation_date"
HPZone_make_valid("case", fields=c("Family name", "Gp")) # should return c("Family_name", "Gp")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HPZone_request.R
\name{HPZone_request_query}
\alias{HPZone_request_query}
\title{Performs a HPZone request with the given parameters.}
\usage{
HPZone_request_query(query, ..., scope = API_env$scope_standard)
}
\arguments{
\item{query}{A GraphQL query to send to the HPZone API. Note that only the actual query is required. (See the examples.)}

\item{...}{Parameters to be passed to [sprintf()]. If empty, the body is not passed through [sprintf()].}

\item{scope}{The desired scope; either standard or extended.}
}
\value{
An object containing the requested data points. This can be in different forms, depending on the request, but is simplified as much as possible.
}
\description{
This function is a convenience wrapper around [HPZone_request_raw()], to facilitate easier coding.
This function integrates [sprintf()] to allow for easier query design, quotes are automatically escaped, and the the result is unlisted to allow for easier access.
See the example for differences with [HPZone_request_raw()]. Note that results are *not* automatically paginated; use [HPZone_request_paginated()] for that.
}
\examples{
# Note the difference between the raw and convenience functions.
# These lines are equal:
\dontrun{
HPZone_request("cases", c("Case_creation_date", "Case_number"),
               where=c("Case_creation_date", ">=", "2025-01-01"))
HPZone_request_query(paste0('cases(where: {',
                              'Case_creation_date: { gte: "2025-01-01" }',
                            '}) {',
                              'items { Case_creation_date, Case_number }',
                             '}'))
HPZone_request_raw(paste0('{"query": "{ cases(where: {',
                            'Case_creation_date: { gte: \\\\"2025-01-01\\\\" }',
                          '}) {',
                            'items { Case_creation_date, Case_number }',
                          '} }"}'))
}
}
\seealso{
[HPZone_request()], [HPZone_request_paginated()], [HPZone_convert_dates()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HPZone_request.R
\name{HPZone_request_paginated}
\alias{HPZone_request_paginated}
\title{Performs a HPZone request with the given parameters.}
\usage{
HPZone_request_paginated(
  query,
  ...,
  n_max = 500,
  scope = API_env$scope_standard,
  verbose = FALSE
)
}
\arguments{
\item{query}{A GraphQL query to send to the HPZone API. Note that keywords 'skip' and 'take' cannot be present in the query; this function will add them.}

\item{...}{Parameters to be passed to sprintf(). If empty, the body is not passed through sprintf().}

\item{n_max}{Maximum number of entries to request per call.}

\item{scope}{The desired scope; either standard or extended.}

\item{verbose}{Can be used to display information about the looping request.}
}
\value{
A data frame containing all the responses gathered from the API. Only the items will be returned.
}
\description{
Convenience wrapper around [HPZone_request_query()]. This function automatically pulls the available number of records, rather than only the first 500.
Note that the current maximum for batching is 500 rows, so increasing n_max is not recommended.
This function is mainly intended for use when the query builder in [HPZone_request()] is insufficient. Usage of [HPZone_request()] is considerably easier otherwise.
An order clause is automatically added if not present to account for the lack of proper automatic sorting in the API.
}
\examples{
\dontrun{
# Note the single quotes to facilitate double quote encapsulation for arguments.
HPZone_request_paginated(
  paste0('cases(where: {',
      'Case_creation_date: { gte: "2025-01-01" }',
    '}) {',
      'items { Case_creation_date, Case_number }',
     '}'))
# Or equal, making use of the sprintf integration:
startdate = "2025-01-01"
fields = c("Case_creation_date", "Case_number")
HPZone_request_paginated(
  'cases(where: { Case_creation_date: { gte: "\%s" } }) { items { \%s } }',
   startdate, stringr::str_c(fields, collapse=", ")
)
}
}
\seealso{
[HPZone_request()], [HPZone_convert_dates()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HPZone_necessary_scope.R
\name{HPZone_necessary_scope}
\alias{HPZone_necessary_scope}
\title{Determines the necessary scope for a given list of fields.}
\usage{
HPZone_necessary_scope(fields, endpoint = "Cases", resolve_fieldnames = TRUE)
}
\arguments{
\item{fields}{List of desired fields from a query.}

\item{endpoint}{The required endpoint.}

\item{resolve_fieldnames}{Whether or not to parse the supplied fields and convert them to valid HPZone field names. (By calling [HPZone_make_valid])}
}
\value{
The required scope as expected by the HPZone API, i.e. "standard" or "extended".
}
\description{
Determines the necessary scope for a given list of fields.
}
\examples{
# these variables do not required an extended scope; desired response = standard
HPZone_necessary_scope(c("Diagnosis", "Case_number", "Entered_by"))
# Family_name requires the extended scope; desired response = extended
HPZone_necessary_scope(c("Diagnosis", "Case_number", "Family_name"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HPZone_setup.R
\name{HPZone_store_credentials}
\alias{HPZone_store_credentials}
\title{Safely stores the HPZone API details, so you don't need to put them in your script. The default OS keyring backend is used through the keyring-package. Values are stored in an encrypted format to prevent harvesting by other applications.
Note that the actual values are not supplied as arguments, but requested using rstudioapi password prompts.}
\usage{
HPZone_store_credentials(client_id = TRUE, client_secret = TRUE)
}
\arguments{
\item{client_id}{True / false: whether the client_id should be stored.}

\item{client_secret}{True / false: whether the client_secret should be stored.}
}
\value{
N/A
}
\description{
Safely stores the HPZone API details, so you don't need to put them in your script. The default OS keyring backend is used through the keyring-package. Values are stored in an encrypted format to prevent harvesting by other applications.
Note that the actual values are not supplied as arguments, but requested using rstudioapi password prompts.
}
\examples{
\dontrun{
# simply execute this line to store credentials
HPZone_store_credentials()
# after use, setup can be ran without arguments:
HPZone_setup()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HPZone_convert_dates.R
\name{HPZone_convert_dates}
\alias{HPZone_convert_dates}
\title{Converts all relevant columns in the dataset to Date types.}
\usage{
HPZone_convert_dates(
  data,
  search = "dat(e|um)|Received_on",
  statdate = "Date_stat"
)
}
\arguments{
\item{data}{A dataset returned by the API.}

\item{search}{Column names to search for. Default: anything containing 'date' or 'datum'.}

\item{statdate}{Desired column name for the date for statistics field.}
}
\value{
An equivalent data.frame with Date types in the correct columns. Additionally, a column called 'Date_stat' is added, see Details.
}
\description{
Based on the available fields, a date for statistics is automatically generated, using the following logic:
If cases: Date_of_onset > Datum_melding_aan_de_ggd > Case_creation_date.
If situations: Start_date > Situation_creation_date
If enquiries: Received_on > Date_closed
}
\examples{
\dontrun{
HPZone_request("cases", "basic", where=c("Case_creation_date", ">", "2025-01-01")) |>
  HPZone_convert_dates()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{HPZone_fields}
\alias{HPZone_fields}
\title{Endpoints and fields available in the HPZone API}
\format{
A tibble containing field names and metadata.
}
\usage{
HPZone_fields
}
\description{
Endpoints and fields available in the HPZone API
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HPZone_setup.R
\name{HPZone_setup}
\alias{HPZone_setup}
\title{Initialisation function to define API credentials. This function must be called before anything else, as the details supplied in this call are required for use.
Note that client_id and client_secret should preferably not be supplied in this call, but rather stored using [HPZone_store_credentials()].}
\usage{
HPZone_setup(
  client_id = NA,
  client_secret = NA,
  standard = "standard",
  extended = "extended",
  token_url = "https://connect.govconext.nl/oidc/token",
  data_url = "https://api.hpzone.nl:8899/Edie"
)
}
\arguments{
\item{client_id}{Client ID as supplied by InFact.}

\item{client_secret}{Client secret as supplied by InFact.}

\item{standard}{Name of the standard scope. Default: "standard"}

\item{extended}{Name of the extended scope. Default: "extended"}

\item{token_url}{Address of the token server.}

\item{data_url}{Address of the data server.}
}
\value{
No return value.
}
\description{
Initialisation function to define API credentials. This function must be called before anything else, as the details supplied in this call are required for use.
Note that client_id and client_secret should preferably not be supplied in this call, but rather stored using [HPZone_store_credentials()].
}
\examples{
# Not recommended:
HPZone_setup("id", "secret")

# Recommended:
\dontrun{
HPZone_store_credentials() # call once
HPZone_setup() # will automatically read stored credentials
}
}
